import { Server } from 'http';
import WebSocket from 'ws';
import { ProxyInfo } from './proxy.ts';
/**
 * websocket代理
 * apiList: [{ path: '/api/router', target: 'https://kevisual.xiongxiao.me' }]
 * @param server
 * @param config
 */
export const wsProxy = (server: Server, config: { apiList: ProxyInfo[] }) => {
  console.log('Upgrade initialization started');

  server.on('upgrade', (req, socket, head) => {
    const proxyApiList: ProxyInfo[] = config?.apiList || [];
    const proxyApi = proxyApiList.find((item) => req.url.startsWith(item.path));

    if (proxyApi && proxyApi.ws) {
      const _u = new URL(req.url, `${proxyApi.target}`);
      const isHttps = _u.protocol === 'https:';
      const wsProtocol = isHttps ? 'wss' : 'ws';
      const wsUrl = `${wsProtocol}://${_u.hostname}${_u.pathname}`;

      const proxySocket = new WebSocket(wsUrl, {
        headers: req.headers,
      });

      proxySocket.on('open', () => {
        socket.on('data', (data) => {
          proxySocket.send(data);
        });

        proxySocket.on('message', (message) => {
          socket.write(message);
        });
      });

      proxySocket.on('error', (err) => {
        console.error(`WebSocket proxy error: ${err.message}`);
        socket.end();
      });

      socket.on('error', () => {
        proxySocket.close();
      });
    } else {
      socket.end();
    }
  });
};
