import { fileProxy, apiProxy, createApiProxy } from '@kevisual/assistant-module/proxy';
import { getCacheAssistantConfig, appDir } from '@kevisual/assistant-module';
import http from 'http';
import { LocalProxy } from './get-local-proxy.ts';
const localProxy = new LocalProxy();
// https://localhost:51015/user/tiptap/
export const proxyRoute = async (req: http.IncomingMessage, res: http.ServerResponse) => {
  const assistantConfig = getCacheAssistantConfig();
  // const { apiList } = assistantConfig;
  const url = new URL(req.url, 'http://localhost');
  const pathname = url.pathname;
  if (pathname.startsWith('/favicon.ico')) {
    res.statusCode = 404;
    res.end('Not Found Favicon');
    return;
  }
  if (pathname.startsWith('/client')) {
    console.log('handle by router');
    return;
  }
  // client, api, v1, serve 开头的拦截
  const apiProxyList = assistantConfig?.apiProxyList || [];
  const defaultApiProxy = createApiProxy(assistantConfig?.pageApi || 'https://kevisual.silkyai.cn');
  const apiBackendProxy = [...apiProxyList, ...defaultApiProxy].find((item) => pathname.startsWith(item.path));
  if (apiBackendProxy) {
    console.log('apiBackendProxy', apiBackendProxy, req.url);
    return apiProxy(req, res, {
      path: apiBackendProxy.path,
      target: apiBackendProxy.target,
    });
  }
  const urls = pathname.split('/');
  const [_, _user, _app] = urls;
  if (!_app) {
    res.statusCode = 404;
    res.end('Not Found Proxy');
    return;
  }
  if (_app && urls.length === 3) {
    // 重定向到
    res.writeHead(302, { Location: `${req.url}/` });
    return res.end();
  }
  const proxyApiList = assistantConfig?.proxy || [];
  const proxyApi = proxyApiList.find((item) => pathname.startsWith(item.path));
  if (proxyApi) {
    console.log('proxyApi', proxyApi, pathname);
    const { user, key } = proxyApi;
    return fileProxy(req, res, {
      path: proxyApi.path, // 代理路径, 比如/root/center
      rootPath: appDir, // 根路径
      indexPath: `${user}/${key}/index.html`, // 首页路径
    });
  }
  const localProxyProxyList = localProxy.getLocalProxyList();
  const localProxyProxy = localProxyProxyList.find((item) => pathname.startsWith(item.path));
  if (localProxyProxy) {
    console.log('localProxyProxy', localProxyProxy, req.url);
    return fileProxy(req, res, {
      path: localProxyProxy.path,
      rootPath: process.cwd(),
      indexPath: localProxyProxy.indexPath,
    });
  }
  console.log('handle by router 404', req.url);
  res.statusCode = 404;
  res.end('Not Found Proxy');
  // console.log('getCacheAssistantConfig().pageApi', getCacheAssistantConfig().pageApi);
  // return apiProxy(req, res, {
  //   path: url.pathname,
  //   target: getCacheAssistantConfig().pageApi,
  // });
};
